#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#pragma ident   "@(#)copy_uncompress_patches.pl 1.1     01/08/14 SMI"
#
# First copy the entire contents of the source patch directory to a
# working directory.  It is in this directory the .tar, .zip, and .Z
# compressed patches are uncompressed.  Any errors that occur are written out
# to an error file.  The patches are not added by this script; the script
# patchadd_local.pl does the actual patch adds using the working directory as
# the patch directory.
# As a last note, the copying and uncompression are forked off by the parent
# process for performance reasons (allows for patch setup to be done in
# parallel by SPM).  Once the child has completed its work, a state file is
# written out.  SPM is awaiting this file as a signal of completion.


require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Common;

# Create the html query object
my $q = new Cluster::Cgi;
my $cluster = new Cluster::Common;

# Print the header
print $q->header();

# Fork so that we can exit to the parent immediately
if (fork()) {
    exit;
}

# Get the arguments
my $patch_path = $q->param("patch_path");
my $error_file = $q->param("error_file");
my $node = $q->param("node");
my $uncompressed_patch_path = $q->param("uncompressed_patch_path");
my $state_file = "/var/cluster/spm/done_copy_uncompress_patches";

# Set the PATH variable
$ENV{"PATH"} = "/usr/sbin:/usr/bin";

# Set the error array
my @errors;

# Clear the old state file by removing it.
if ( -f $state_file ) {
    `rm -fr $state_file`;
    if ($? != 0) {
        $cluster->syslog("rm -fr $state_file FAILED!");
        push (@errors, "On $node, rm -fr $state_file FAILED");
    }
}

# Clear the old error file by removing it.
if ( -f $error_file ) {
    `rm -fr $error_file`;
    if ($? != 0) {
        $cluster->syslog("rm -fr $error_file FAILED!");
        push (@errors, "On $node, rm -fr $error_file FAILED");
    }
}

# Create the working directory for the patches, if needed
if ( ! -d $uncompressed_patch_path ) {
    `mkdir -p $uncompressed_patch_path`;
    if ($? != 0) {
        $cluster->syslog("mkdir -p $uncompressed_patch_path FAILED!");
        push (@errors, "On $node, mkdir -p $uncompressed_patch_path FAILED");
    }
} else {
    # Remove any files in this directory that might have been created by
    # a previous attempt to uncompress patches.
    `rm -fr $uncompressed_patch_path/*`; # comment close is workaround for bug #4398335 */
    if ($? != 0) {
        $cluster->syslog("rm -fr $uncompressed_patch_path/* FAILED!");
        push (@errors, "On $node, rm -fr $uncompressed_patch_path/* FAILED");
    }
}
# The uncompressed_patch_path directory is created and is empty, so
# copy from the patch location to the working directory
`cp -R $patch_path/* $uncompressed_patch_path`; # comment close is workaround for bug #4398335 */
if ($? != 0) {
    $cluster->syslog("cp -R $patch_path/* $uncompressed_patch_path FAILED!");
    push (@errors, "On $node, cp -R $patch_path/* $uncompressed_patch_path FAILED");
}

# If there were any errors up to this point, then skip the uncompression stuff
if ( ! @errors ) {
    # Uncompress the following compression formats: .Z .zip .tar
    my @format_extension = ("Z","zip","tar");
    my $uncompress_command;

    foreach (@format_extension) {
        my @uncompress = `ls $uncompressed_patch_path/*.$_ 2>/dev/null`; # comment close is workaround for bug #4398335 */
        if ( $_ eq "Z" ) {
            $uncompress_command = "uncompress";
        } elsif ($_ eq "zip") {
            $uncompress_command = "unzip";
        } else {
            $uncompress_command = "tar xf";
        }
        # Uncompress all of them
        foreach my $patch (sort @uncompress) {
            chomp $patch;
            `cd $uncompressed_patch_path ; $uncompress_command $patch`;
            if ($? != 0) {
                $cluster->syslog("$uncompress_command of $patch FAILED");
                push (@errors, "On $node, $uncompress_command of $patch FAILED");
            } else {
                $cluster->syslog("$uncompress_command of $patch SUCCEEDED");
            }
        } # foreach $patch
    } # foreach format type (.Z, .zip, .tar )
} # ! @errors

# Touch the file that indicates we're done
`touch $state_file`;
if ($? != 0) {
    $cluster->syslog("touch $state_file FAILED!");
    push (@errors, "touch $state_file FAILED!");
}

# If there were any errors in this script, then print them to the touched file.
if ( @errors) {
    open(FILE, ">$error_file");
    foreach my $e (@errors) {
        print FILE "$e\n";
    }
    close(FILE);
}
# Exit completely
CORE::exit(0);
