#!/usr/perl5/bin/perl
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)step6.pl	1.39	01/08/14 SMI"
#
# Installation page 6

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Install;
use Sun::Solaris::Utils qw(gettext);

# Set autoflush
$| = 1;

# Set pfexec path
my $PFEXEC = "/usr/bin/pfexec";

# Remove the BASEDIR environment variables
delete @ENV{'BASEDIR', 'CLIENT_BASEDIR'};

# Create the html query object
my $q = new Cluster::Cgi;

# Create a install object
my $install = new Cluster::Install;

# Start the html document
$install->header($q,
		 gettext("Sun Cluster Installation &gt; Installation in Progress"));
print $q->p({class => "action-window-normal-text"});

# Print info
print gettext("The cluster is now being installed and configured.  " .
  "This process may take up to 90 minutes depending on the options you " .
  "have selected. When the installation has finished, this window will " .
  "automatically reload with the SunPlex Manager monitoring and " .
  "configuration GUI.");

print $q->p({class => "action-window-normal-text"});
print $q->img({	SRC => "/images/warning_16.gif",
		WIDTH => 16,
		HEIGHT => 16,
		ALT => "Warning"});
print " ";
print gettext("Do not close this browser window or modify the URL while " .
    "the installation is in progress.");
print $q->table({ CLASS => "action-table",
		  WIDTH => "97%",
		  BORDER => 0,
		  CELLSPACING => 0,
		  CELLPADDING => 0,
		  ALIGN => "center",
	  });
print $q->start_Tr({HEIGHT => 10});
print $q->start_td({COLSPAN => 2});
print $q->img({SRC => "/images/dot.gif", ALT => "", HEIGHT => 10});
print $q->end_td(), $q->end_Tr(), "\n";

print $q->start_Tr({HEIGHT => 1});
print $q->start_td({COLSPAN => 2, CLASS => "action-line-row"});
print $q->img({SRC => "/images/dot.gif", ALT => "", HEIGHT => 1});
print $q->end_td(), $q->end_Tr(), $q->end_table(), "\n";

# Setup the needed variables
my $cluster_name = $q->param('cluster_name');
my $cluster_size = $q->param('cluster_size');
my $cdrom_path = $q->param('cdrom_path');
my $ds_cdrom_path = $q->param('ds_cdrom_path');
my $solaris_cdrom_path = $q->param('solaris_cdrom_path');
my $install_nfs = $q->param('install_nfs');
my $nfs_address = $q->param('nfs_address');
my $patch_path = $q->param('patch_path');
my $install_apache = $q->param('install_apache');
my $apache_address = $q->param('apache_address');
my $run_sccheck = $q->param('run_sccheck');
my $retry_patches = $q->param('retry_patches');
my $uncompressed_patch_path = $install->get_uncompresspatchdir();

# Get the list of nodes we're installing
my @nodelist = $install->get_nodelist($q);
my $clustersize = $#nodelist + 1;

# Create a nodelist string
my $nodestring = join ',', @nodelist;

# Get the log directory to use
my $logdir = $install->get_logdir();

# Check if Sun Cluster is already installed
my $sc_installed=0;
foreach my $node (@nodelist) {
    if ($install->get_install_state($node, "DONE_SCINSTALL_I") eq 'YES') {
        $install->syslog("Sun Cluster has already been installed on $node.");
        $sc_installed++;
    }
}
if ($sc_installed == $clustersize) {
    # Skip the Sun Cluster installation as it already has been done.
    # Print message for the user
    print $q->p({CLASS => "action-window-normal-text"});
    print gettext("Sun Cluster packages already installed...\n");
} else {
    # Install Sun Cluster software packages
    print $q->p({CLASS => "action-window-normal-text"});
    print gettext("Installing Sun Cluster software packages...\n");

    foreach my $node (@nodelist) {
	my $pkgadd = "/cgi-bin/installation/scinstall.pl";
	$pkgadd .= "?cdrom_path=" . $q->param('cdrom_path') .
		"&mode=install_sccheck";
	$install->syslog("Running 'scinstall -i' on $node");
	$install->get_http($node,
			   $install->get_http_port(),
			   $pkgadd);
    }
    # Wait for the scinstall -i
    my $pkgadd_complete = 0;
    while ($pkgadd_complete == 0) {
	$pkgadd_complete = 1;
	foreach my $node (@nodelist) {
		if ($install->get_install_state($node,
			"DONE_SCINSTALL_I") ne 'YES') {
			$install->syslog("'scinstall -i' has NOT " .
					 "finished on $node");
			$pkgadd_complete = 0;
			last;
		} else {
			$install->syslog("scinstall -i' has " .
					 "finished on $node");
		}
	}
	sleep(5);
    }
    print gettext("&nbsp;&nbsp;DONE");
}

# Run sccheck to validate the cluster configuration
if ($run_sccheck ne "false") {

	print $q->p({CLASS => "action-window-normal-text"});
	print gettext("Running sccheck to validate the " .
		      "cluster configuration...\n");

	# Run sccheck on all nodes to validate the configuration
	my $sccheck_cmd = "/usr/cluster/bin/sccheck -h $nodestring";
	$install->syslog("Running: $sccheck_cmd");	
	my @errors = `$PFEXEC $sccheck_cmd`;
	my $ret = $? / 256;
	$install->syslog("sccheck returned $ret");

	# If we have sccheck errors, print them out
	if ($ret != 0) {
		foreach my $error (@errors) {
			$install->syslog($error);
		}

		my $query_string = $q->get_cluster_query_string();
		my $retry_jscript = "self.location=\"step6.pl$query_string\"";
		my $continue_jscript = $retry_jscript . 
			" + \"&run_sccheck=false\"";

		# Print the message for a sccheck warning
		if ($ret == 1) {
			$install->start_message_table($q, 
			        "/images/warning_32.gif",
   		                gettext("Cluster Validation Warning"),
		                gettext("Cluster Validation Warning"));

			print $q->p({ CLASS => "message-description-text"},
				    gettext("The command:"));

			print $q->p({ CLASS => "message-command-text"},
				    $sccheck_cmd);

			print $q->p({ CLASS => "message-description-text"},
				    gettext("Returned the following " .
					    "warning:"));
		} else {
			$install->start_message_table($q, 
			        "/images/error_32.gif",
   		                gettext("Cluster Validation Failed"),
		                gettext("Cluster Validation Failed"));

			print $q->p({ CLASS => "message-description-text"},
				    gettext("The command:"));

			print $q->p({ CLASS => "message-command-text"},
				    $sccheck_cmd);

			print $q->p({ CLASS => "message-description-text"},
				    gettext("Failed with the following " . 
					    "error:"));
		}

		print $q->start_p({ CLASS => "message-command-text"});
		foreach my $error (@errors) {
			print $error, $q->br();
		}
		print $q->end_p();

		$install->end_message_table($q);
		print $q->p({class => "action-window-normal-text"});
		print gettext("Either fix the indicated problems and " .
			      "press the \"Retry Check\" button or press " .
			      "the \"Continue Installation\" button to " .
			      "continue installing the cluster with the " .
			      "existing errors and/or warnings.");

		$install->start_button_table($q);

		print $q->start_form();
		print $q->button({ value   => gettext("  < Back  "),
				   onClick => "history.back()"
				   });
		print "&nbsp";
		print $q->button({ value => gettext("  Retry Validation  "),
				   onClick => $retry_jscript
				   });
		print "&nbsp";
		print $q->button({ value => gettext("  Continue " .
						    "Installation > "),
				   onClick => $continue_jscript
				   });

		$install->end_button_table($q, "Cancel");
		print $q->end_form();
    
		# End the html document
		$install->footer($q);

		# Exit gracefully
		exit(0);
	}

	print gettext("&nbsp;&nbsp;DONE");
}

# After going through the sccheck stuff, write the nodelist and
# clustersize to all of the nodes. Also, write state files to make
# sure that the correct rc scripts get run after installation.
foreach my $node ($install->get_nodelist($q)) {
    my $nodefile = "$logdir/nodelist";
    my $sizefile = "$logdir/clustersize";
    my $quorumfile = "$logdir/setup_quorum_flag";
    $install->appendfile_remote($node, $sizefile, $clustersize);
    $install->appendfile_remote($node, $nodefile, $nodestring);
    $install->appendfile_remote($node, $quorumfile, "1");
}

# Check if SDS is already installed
my $sds_installed=0;
foreach my $node (@nodelist) {
    if ($install->get_install_state($node, "DONE_SDS") eq 'YES') {
        $install->syslog("SDS has already been installed on $node.");
        $sds_installed++;
    }
}

# Install SDS
if ($q->param('install_sds') eq 'yes' && ($sds_installed == $clustersize)) {
    # Skip the SDS installation as it already has been done.
    # Print message for the user
    print $q->p({CLASS => "action-window-normal-text"});
    print gettext("Skipping installation of Solstice DiskSuite...\n");
} elsif ($q->param('install_sds') eq 'yes' && ($sds_installed!=$clustersize)) {

    # Common vars
    my ($ret, $error);
    $error = 0;

    # Print message for the user
    print $q->p({CLASS => "action-window-normal-text"});
    print gettext("Installing Solstice DiskSuite...\n");

    # 1. Call cgi-bin/installation/sds/installation in parallel on all nodes
    foreach my $node ($install->get_nodelist($q)) {

	    # Setup state files
	    my $devicefile = "$logdir/setup_devices_flag";
	    my $sdsfile = "$logdir/setup_sds_flag";
	    $install->appendfile_remote($node, $devicefile, "1");
	    $install->appendfile_remote($node, $sdsfile, "1");

	    my $installation = "/cgi-bin/installation/sds/installation.pl";
	    $installation .= "?solaris_cdrom_path=$solaris_cdrom_path";
	    $install->syslog("Installing Solstice DiskSuite on $node");
	    
	    $install->get_http($node,
			       $install->get_http_port(),
			       $installation);
    }

    # 2. Wait for the SDS installation to finish on all nodes
    my $sds_complete = 0;

    while ($sds_complete == 0) {
	$sds_complete = 1;

	foreach my $node (@nodelist) {
	    if ($install->get_install_state($node, "DONE_SDS") ne 'YES') {
		$install->syslog("Solstice DiskSuite has NOT finished on $node");
		$sds_complete = 0;
		last;
	    } else {
		$install->syslog("Solstice DiskSuite has finished on $node");
	    }
	}

	sleep(5);
    }

    # 3. Call cgi-bin/installation/sds/get_ctd_disks.ksh serially on all nodes,
    #    Redirecting the its output to a file corresponding to each node.
    foreach my $node ($install->get_nodelist($q)) {
	my ($status, @output, $ctd);
	
	$ctd = "/cgi-bin/installation/sds/get_ctd_disks.pl";
	$install->syslog("Running get_ctd_disks.ksh disks on $node");
	@output = $install->get_http($node, $install->get_http_port(), $ctd);
	open OUTPUT, ">$logdir/ctd.$node.$$";

	foreach my $line (@output) {
		print OUTPUT "$line\n";
	}
	close OUTPUT;
	
	foreach my $line (@output) {
	    $install->syslog($line);
	}
    }

    # 4. Call cgi-bin/installation/sds/config_sds on one node. 
    #    For each node, three arguments will be given to config_sds:
    #    < node datafile metadb_slice >
    #        node : The node name
    #        datafile : The files generated when get_ctd_disks.pl was previously
    #                    run on the node.
    #        metadb_slice : This node's dedicated rootdisk partition set aside
    #	                    for metadb creation.  Placing the /sds filesystem 
    #			    is how the partition is dedicated for the metadb 
    #			    replica.  The metadb_slice format is cNtNdNsN 
    #			    (e.g. c0t0d0s4 ).
    #    config_sds will write the rc scripts to the appropriate nodes.
    # If any metadb_slice cannot be determined, then the config_sds script
    # will not be run and SDS installation will be skipped entirely.
    
    # Construct the call to config_sds
    my $config = "/opt/SUNWscvw/cgi-bin/installation/sds/config_sds";
    foreach my $node ($install->get_nodelist($q)) {
        my $metadb_slice = $install->get_metadb_slice($node);
        if ( defined $metadb_slice ) {
             $install->syslog("Successfully located the /sds partition ($metadb_slice) on the rootdisk of node $node.");
   	     $config .= " $node $logdir/ctd.$node.$$ $metadb_slice";
        } else {
             $install->syslog("Could not locate the /sds partition on the rootdisk of node $node that is required for Solstice DiskSuite installation.");
	     $error++;
	}
    }
    # Check to see if errors were encountered when locating the /sds partition.
    if ( $error > 0 ) {
	$install->syslog("Skipping Solstice DiskSuite installation.");
        print gettext("&nbsp;&nbsp;Unexpected problems occurred - Skipping Solstice DiskSuite Installation...");
    } else {
        # Run the config_sds binary
        $install->syslog("Running: $config");
        my @output = `$PFEXEC $config`;
        $error += $?;
        # Report errors from the execution
        foreach my $line (@output) {
        	$install->syslog($line);
        }
    
        # 5. Open the appendfile.dat file to create the datafiles that contain 
	# the information needed for metadb/metaset/metadevice creation on each 	# node.  The appendfile.dat file was created by config_sds.
        if (open(FILE, "$logdir/appendfile.dat")) {
    	    while (my $line = <FILE>) {
	        chomp $line;
	        my ($host, $filename, @string, $space);
	        ($host, $filename, @string) = split / /, $line;
	        my $fileline = join ' ', @string;
	        $install->appendfile_remote($host, $filename, $fileline);
	    }
	    close(FILE);
        } else {
    	    $error++;
        }

        # 6. Save the vfstab entry in a datafile on all nodes
        foreach my $node ($install->get_nodelist($q)) {
            $install->append_vfstab_entry($node);
        }

        # Final reporting of status
        if ($error != 0) {
	    $install->syslog("config_sds FAILED with return value $error");
        }

        print gettext("&nbsp;&nbsp;DONE");
    } # End config_sds execution 
} # End sds install

# Install patches if we've defined a patch directory path
if (defined $patch_path && $patch_path ne '' &&
        $retry_patches ne "false" ) {
    print $q->p({CLASS => "action-window-normal-text"});
    print gettext("Adding patches...\n");

    my $patchadd_error_log="/var/cluster/spm/patchadd_error_log";
    # Call cgi-bin/installation/patchadd_local.pl on all nodes
    # and pass in the location of the uncompressed patches
    foreach my $node (@nodelist) {
	my $patchadd = "/cgi-bin/installation/patchadd_local.pl";
	$patchadd .= "?uncompressed_patch_path=$uncompressed_patch_path".
             "&patchadd_error_log=$patchadd_error_log";
	$install->syslog("Installing patches on $node");
	
	my @output = $install->get_http($node, $install->get_http_port(),
	      $patchadd);
	
	foreach my $line (@output) {
	    print $line;
	}
    }
    
    # Wait for the patch installation to finish on all nodes
    my $patch_complete = 0;
    my $patch_errors_nodes = 0;
    foreach my $node (@nodelist) {
        # redo comes here
        if ($install->get_install_state($node, "DONE_PATCHES") ne 'YES') {
            $install->syslog("PATCH ADDING has NOT finished on $node");
            sleep(10);
            redo;
        } else {
            $install->syslog("PATCH ADDING has finished on $node");
            # Check each node's error log.  If there were errors when
            # adding patches, then halt the installation and prompt users
            # to either resolve any patch conflicts by hand, ignore the patch
	    # add errors, go back to the previous screen, or cancel.
            my @patch_errors = $install->check_patch_errors($node,
		$patchadd_error_log);
            if (@patch_errors) {
                # There were errors when adding certain patches!
                $patch_errors_nodes++;
                if ( $patch_errors_nodes == 1 ) {
		    $install->start_message_table($q, "/images/error_32.gif",
		        gettext("Error"), gettext("Patch Installation"),
		        gettext("The following errors occurred during patch " .
		            "installation:"));
		}
                foreach my $p_e ( @patch_errors ) {
                     # Print message for the user
                     print $q->p({CLASS => "message-command-text"}, $node,
			$p_e);
                     $install->syslog("$node $p_e");
                     $q->br();
                }
            }
        }
    }
    # End the message table if any patch errors were encountered.
    if ( $patch_errors_nodes != 0 ) {
        $install->end_message_table($q);
    }

    # Remove the state file in case that another attempt to add patches takes
    # place.
    foreach my $node (@nodelist) {
	# Remove the DONE_PATCHES state file.
        $install->remove_done_patches_state_file($node);
    }

    # Done with all the nodes
    # If there were any errors seen on any node, then print them to the user
    if ( $patch_errors_nodes > 0) {
	my $query_string = $q->get_cluster_query_string();
	my $retry_jscript = "self.location=\"step6.pl$query_string\"".
		" + \"&run_sccheck=false\"";
	my $continue_jscript = $retry_jscript . 
		" + \"&retry_patches=false\""; 
        # Prompt the user on their next course of action.
        print $q->p({class => "action-window-normal-text"});
        print gettext("The patches that did not encounter errors have been successfully added.  However, some patches experienced errors during patch addition.");
	print "<UL>";
	print "<LI> ".gettext("To complete patch installation, manually resolve the above patch installation errors, then click 'Retry Adding Patches'. ");
	print "<LI>".gettext("To ignore the above patch errors and proceed without the above patches, click 'Continue with Installation'.");
	print "<LI>".gettext("To return to the previous screen, click 'Back'.");
	print "<LI>".gettext("To cancel installation, click 'Cancel'.");
	print "</UL>";

        print $q->start_form({ action => '/cgi-bin/installation/step6.pl' });
        # Print the hidden cluster fields
        $q->print_hidden_cluster_fields();
        $install->start_button_table($q);
	print $q->button({ value   => gettext(" < Back  "),
			   onClick => "history.back()"
			   });
	print "&nbsp";
	print $q->button({ value => gettext("  Retry Adding Patches  "),
			   onClick => $retry_jscript
			   });
	print "&nbsp";
        print $q->button({ value   => gettext(" Continue with Installation "),
			   onClick => $continue_jscript
        		});
	print "&nbsp";
        $install->end_button_table($q, "cancel");
        print $q->end_form();
        # End the html document
        $install->footer($q);
        # end gracefully if errors were found
        exit(0);
    }
    # Print some blank lines.
    print gettext("&nbsp;&nbsp;DONE");
} # End patch install


# Ie we're installing HA NFS as a dataservice, then 
#	- Create an rc script that will install the NFS package on each
#		node.
#	- Check to make sure that we have a metaset 'mirror-1' created
#	- Find the node that is responsible for mirror-1 creation and
#		copy an rc script that will configure HA NFS using mirror-1.
#
# NOTE : It is possible that we are installing Solstice DiskSuite, but we
# do NOT find enough shared disks to make a mirrored metadevice.  If this is the
# case, then we cannot create a global filesystem which is a prerequisite 
# for HA NFS.
if ($install_nfs eq 'yes' || $install_apache eq 'yes') {

    # Choose which data services to install
    print $q->p({CLASS => "action-window-normal-text"});
    if ($install_apache eq 'yes') {
	if ($install_nfs eq 'yes') {
	    print gettext("Installing Apache and NFS...\n");
	} else {
	    print gettext("Installing Apache...\n");
	}
    } else {
	print gettext("Installing NFS...\n");
    }
    
    foreach my $node ($install->get_nodelist($q)) {
	my ($find_mirror_metaset, $result, $copy_nfs_scripts, 
		$nfs_installation);
	$find_mirror_metaset = "/cgi-bin/installation/nfs/find_mirror_metaset.pl";
	$nfs_installation = "/cgi-bin/installation/nfs/nfs_installation.pl" .
		"?cdrom_path=" . $q->param('cdrom_path') .
		"&ds_cdrom_path=" . $q->param('ds_cdrom_path') .
		"&solaris_cdrom_path=" . $q->param('solaris_cdrom_path'); 

	# Choose which data services to install
	if ($install_nfs eq 'yes') {
	    $nfs_installation .= "&install_nfs=yes";

	    # Write the logical hostname to all nodes
	    $install->appendfile_remote($node,
					"$logdir/logical_host",
					$nfs_address);
	}

	if ($install_apache eq 'yes') {
	    $nfs_installation .= "&install_apache=yes&apache_address=" . 
		$q->param("apache_address");

	    # Write the shared address to all nodes
	    $install->appendfile_remote($node,
					"$logdir/shared_address",
					$apache_address);
	}

	# Write the path of the data services cdrom
	if ($install_apache eq 'yes' || $install_nfs eq 'yes') {
		$install->appendfile_remote($node,
					    "$logdir/ds_cdrom_path",
					    $q->param('ds_cdrom_path'));
		$install->appendfile_remote($node,
					    "$logdir/setup_ds_packages_flag",
					    1);
	}

	# Write a rc script that will install the SUNWscnfs package on each
	# node of the cluster.
	($result) = $install->get_http($node,
				       $install->get_http_port(), 
				       $nfs_installation);
	chomp $result;
	if ($result eq "OK") {
		$install->syslog("Installing NFS packages on $node...");
	} else {
		print $q->start_b();
		print sprintf(gettext("Installation of NFS and/or Apache packages on %s failed!\n"), $node);
		print $q->br(), $q->end_b();
		$install->syslog("Installation of NFS and/or Apache packages on $node failed.");
	}

	# Only copy the rc script copy_nfs_scripts to the node that will
	# configure the HA NFS resource.
	($result) = $install->get_http($node, $install->get_http_port(), 
				$find_mirror_metaset);
	chomp $result;
	if ($result eq "FOUND") {
		$install->syslog("Configuring Apache/NFS on $node.");

		# Write the setup nfs flag to all nodes
		$install->appendfile_remote($node,
					    "$logdir/setup_nfs_flag",
					    "1");


		# Write the setup apache flag to all nodes
		$install->appendfile_remote($node,
					    "$logdir/setup_apache_flag",
					    "1");
	}
    }

    print gettext("&nbsp;&nbsp;DONE");
}

# Create loop variables
my $loopnode;
my $loopinstall;
my $nodenametocall;
my $command;

# Print message instructing the user to reload after the install
print $q->p({CLASS => "action-window-normal-text"});
print gettext("Configuring Sun Cluster and rebooting nodes...");

# SCINSTALL here!
for ($loopinstall = 1; $loopinstall <= $cluster_size; $loopinstall++) {
	$install->syslog("Installing and rebooting node $loopinstall");
	$nodenametocall = $q->param("node_$loopinstall.name");
	$command = $install->call_remote_scinstall($nodenametocall, 
						   $loopinstall, 
						   $q);
}

# Print the applet to wait and reload
print $q->applet({ codebase => "/",
		   code => "InstallStatus.class",
		   width => 1,
		   height => 1
		   });
print $q->end_applet();

# End the html document
$install->footer($q);
