#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)rg_add.pl	1.13	01/08/14 SMI"
#
# Present forms for adding a resource group
#

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Rg;
use Sun::Solaris::Utils qw(gettext);

# Include needed Perl modules
use Cluster::Rg;
use Cluster::Cgi;
use Cluster::common;

# Create objects
my $q = new Cluster::Cgi;
my $rg_obj = new Cluster::Rg;

# Get the nodelist
my @nodelist = $rg_obj->get_nodelist();

# Get the resource group list
my @rglist = $rg_obj->get_rglist();

$rg_obj->header($q, [gettext("Resource Groups"), "/cgi-bin/rg/rg_view.pl",
	gettext("Create Resource Group")], "rg");

print '<SCRIPT LANGUAGE="JavaScript">
<!-- script hiding
// based on http://javascript.internet.com/forms/menu-swapper.html

function horizontal_shift(fbox,tbox) {
  for(var i=0; i<fbox.options.length; i++) {
      if(fbox.options[i].selected
           && fbox.options[i].value != ""
           && fbox.options[i].value != "ignoreMe") {

	// ignoreMe is last entry: save it
	var tboxlen = tbox.options.length;
	var iotext = tbox.options[tboxlen-1].text;
	var iovalue = tbox.options[tboxlen-1].value;

	// write new entry on top of old ignoreMe
        tbox.options[tboxlen-1].text = fbox.options[i].text;
        tbox.options[tboxlen-1].value = fbox.options[i].value;

	// insert new last option to hold ignoreMe
        var no = new Option();
        tbox.options[tboxlen] = no;
        tbox.options[tboxlen].text = iotext;
        tbox.options[tboxlen].value = iovalue;

        fbox.options[i].value = "";
        fbox.options[i].text = "";
      }
  }
  BumpUp(fbox);
  // let selection drop down except onto spacer
  var fboxlen = fbox.options.length;
  if(fbox.options[fboxlen - 1].value == "ignoreMe") {
    fbox.options[fboxlen - 1].selected = false;
  }
} // horizontal_shift

function BumpUp(box)  {
  for(var i=0; i<box.options.length; i++) {
    if(box.options[i].value == "")  {
      for(var j=i; j<box.options.length-1; j++)  {
        box.options[j].value = box.options[j+1].value;
        box.options[j].text = box.options[j+1].text;
      }
      var ln = i;
      break;
    }
  }
  if(ln < box.options.length)  {
    box.options.length -= 1;
    BumpUp(box);
   }
} // BumpUp

function vertical_shift(box,down) {
  for(var i=0; i<box.options.length; i++) {
      if(box.options[i].selected
           && box.options[i].value != ""
           && box.options[i].value != "ignoreMe") {
	if(down == "1") {
	    swap(box,i,i+1);
	    if(box.options[i+1].value != "ignoreMe") {
	        box.options[i].selected = false;
		box.options[i+1].selected = true;
	    }
	} else {
	    if(i>0) { // IE Solaris fails the >=0 bounds checking in swap()
		swap(box,i,i-1);
  	        box.options[i].selected = false;
		box.options[i-1].selected = true;
	    }
	}
	break;
      }
  }
} // vertical_shift

function swap(box,pos1,pos2) {
    if(box.options[pos1].value != "ignoreMe"
        && box.options[pos2].value != "ignoreMe"
	&& pos1 <= box.options.length
	&& pos2 <= box.options.length
	&& pos1 >= 0
	&& pos2 >= 0 ) {
	var savetext =  box.options[pos1].text;
	var savevalue = box.options[pos1].value;

	box.options[pos1].text = box.options[pos2].text;
	box.options[pos1].value = box.options[pos2].value;

	box.options[pos2].text = savetext;
	box.options[pos2].value = savevalue;
    }
} // swap

function bundleListAsInput(box,outval) {
  outval.value = "";
  for(var i=0; i<box.options.length; i++) {
      if(box.options[i].value != ""
          && box.options[i].value != "ignoreMe") {
        if(outval.value == "") {
          outval.value = box.options[i].value;
        } else {
          outval.value = outval.value + "," + box.options[i].value;
        }
      }
  }  
} //bundleListAsInput

// End script hiding -->
</SCRIPT>';

print $q->_script($q->yoke_jscript("rg"));

# construct the available & selected node scrolling lists
my $spacer = "____________________";
my @clustnodeslist = (@nodelist,"ignoreMe"); # noI18N
my @availnodeslist;
my @selectednodeslist;
my %nodelistlabels;
foreach my $node (@nodelist) {
  $nodelistlabels{$node} =  $node;
}
$nodelistlabels{"ignoreMe"} = $spacer; # noI18N

# preserve selected nodes list from simple page if going to detailed page
# else construct default lists with non-selectable (ignorable  spacers

my $primaries_list_string;
if (defined($q->param('primaries_list'))) {
  $primaries_list_string = $q->param('primaries_list');
  my @primlist = split /,/, $primaries_list_string;

  # avail == clusternodes - already selected nodes
  my $tmpnode;
  foreach $tmpnode (@nodelist) {
  if (index ($primaries_list_string,$tmpnode) == -1) {
     @availnodeslist = (@availnodeslist, $tmpnode);
   }
  } # foreach
  @availnodeslist =    (@availnodeslist,"ignoreMe");
  @selectednodeslist = (@primlist,"ignoreMe");
} else {
  @availnodeslist = (@clustnodeslist);
  @selectednodeslist = ("ignoreMe");
}

if (defined($q->param('doit'))) {
    # Create the resource group

    my ($rg) = $q->param('rg');

    if ($rg eq "") {
	&errmsg("Resource group name must be specified");
	$rg_obj->view_button($q);
	$Cluster::common::q = $q;
	$rg_obj->footer($q);
	exit;
    }

    my ($cmd, $err, $ret, @rg_dep_list, $rg_dep_string);

    $cmd = "-a -g $rg";
    
    if (defined($q->param('desired_primaries')) &&
	    $q->param('desired_primaries') != 1) {
	$cmd .= " -y desired_primaries=".$q->param('desired_primaries');
    }
    
    if (defined($q->param('maximum_primaries')) &&
	    $q->param('maximum_primaries') != 1) {
	$cmd .= " -y maximum_primaries=".$q->param('maximum_primaries');
    }
    
    if (defined($q->param('rg_desc')) && $q->param('rg_desc') ne "") {
	$cmd .= " -y RG_description=\"".$q->param('rg_desc')."\"";
    }
    
    if (defined($q->param('rg_pathprefix')) &&
	    $q->param('rg_pathprefix') ne "/") {
	$cmd .= " -y Pathprefix=\"".$q->param('rg_pathprefix')."\"";
    }

    if (defined($q->param('failback')) && $q->param('failback') eq "true") {
	$cmd .= " -y Failback=True";
    }

    if (defined($q->param('pingpong')) && $q->param('pingpong') != 3600) {
	$cmd .= " -y Pingpong_interval=".$q->param('pingpong');
    }

    if (defined($q->param('globalresources')) &&
	    $q->param('globalresources') ne "all") {
	$cmd .= " -y Global_resources_used=\"\"";
    }

    if (defined($q->param('netdependencies')) &&
	    $q->param('netdependencies') eq "false") {
	$cmd .= " -y Implicit_network_dependencies=False";
    }

    if (defined($q->param('rg_dependencies'))) {
	@rg_dep_list = $q->param('rg_dependencies');
       
	foreach $rg (@rg_dep_list) {
	    if (!defined($rg_dep_string)) {
		$rg_dep_string = $rg;
	    } else {
		$rg_dep_string .= ",$rg";
	    }
	}
	
	$cmd .= " -y rg_dependencies=".$rg_dep_string;
    }

    if (defined($q->param('primaries_list'))) {
      $primaries_list_string = $q->param('primaries_list');
	if(length ($primaries_list_string) > 0) {
	   $cmd .= " -h $primaries_list_string ";
        }
    }

    $ret = &runsuid("scrgadm $cmd");

    if ($ret != 0) {
	&errmsg(sprintf(gettext("Resource group %s could not be created."),
		$rg));
	print "$result\n";


    } else {
	&okmsg(sprintf(gettext("Resource group %s successfully created."),
		$rg));
	&reloadmenu($q);
	print $q->_script($q->yoke_jscript("rg-$rg"));
    }
    $rg_obj->view_button($q);
    $Cluster::common::q = $q;
} elsif (defined($q->param('full'))) {
    # Display full information

    # Start the form
    print $q->start_form({ ACTION => "/cgi-bin/rg/rg_add.pl",
			   METHOD => "POST"
			   });

    # Start the action table
    $q->start_action_table();

    # General properties
    $q->start_action_tr();
    $q->start_action_td({COlSPAN => 2});
    print $q->CGI::div({CLASS => "action-window-label-text"}, 
	gettext("General Properties:"));
    $q->end_action_td();
    $q->end_action_tr();

    # Input rg name
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Resource Group Name:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME => "rg",
		      SIZE => 10
		      });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Description:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME => "rg_desc",
		      SIZE => 10
		      });
    $q->end_action_td();
    $q->end_action_tr();


    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Pathprefix Directory:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME => "rg_pathprefix",
		      SIZE => 10,
		      DEFAULT => "/",
		      });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr();
    $q->start_action_td({COlSPAN => 2});
    print $q->CGI::div({CLASS => "action-window-label-text"},
	gettext("Nodes:"));
    $q->end_action_td();
    $q->end_action_tr();

    # Input primaries
    $q->start_action_tr();
    $q->start_action_td({ VALIGN => "top" });
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Primaries List:"));
    $q->end_action_td();
    $q->start_action_td();

	# print "<br> -- WIDGET START DETAILS 2 --<br>";

    print $q->start_table({ BORDER => 0 });
    print $q->start_Tr();
    print $q->start_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Available Nodes"));
    print $q->end_td();

    print $q->start_td();
    print $q->end_td();

    print $q->start_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Selected Nodes"));
    print $q->end_td();
    print $q->end_Tr();

    print $q->start_Tr();
    print $q->start_td();
    # The following hidden input carries data from the selected nodes list
    print $q->input ({ type => "hidden",
		       name => "primaries_list" });

    print $q->scrolling_list({ single => "true",
			       size => "9",
			       name => "availnodes",
			       values => \@availnodeslist,
			       labels => \%nodelistlabels
			     });
    print $q->end_td();
    print $q->start_td();

    print $q->input({ type => "button",
		      value => gettext("   Add >>  "),
		      onclick => "horizontal_shift(this.form.availnodes, this.form.selnodes);
				  bundleListAsInput(this.form.selnodes,this.form.primaries_list)" });
		      print $q->br();
    print $q->input({ type => "button",
		      value => gettext("<< Remove"),
		      onclick => "horizontal_shift(this.form.selnodes, this.form.availnodes);
				  bundleListAsInput(this.form.selnodes,this.form.primaries_list)" });
    print $q->end_td();
    print $q->start_td();
    print $q->scrolling_list({ single => "true",
			       size => "9",
			       name => "selnodes",
			       values => \@selectednodeslist,
			       labels => \%nodelistlabels
			     });
    print $q->end_td();

    print $q->start_td();

    print $q->input({ type => "button",
		      value => gettext("  Up  "),
		      onclick => "vertical_shift(this.form.selnodes,0);
				  bundleListAsInput(this.form.selnodes,this.form.primaries_list)" });
		      print $q->br();
    print $q->input({ type => "button",
		      value => gettext("Down"),
		      onclick => "vertical_shift(this.form.selnodes,1);
				  bundleListAsInput(this.form.selnodes,this.form.primaries_list)" });
    print $q->end_td();

    print $q->end_Tr();
    print $q->end_table();

	# print "<br> -- WIDGET DETAILS END --<br>";

    $q->end_action_td();
    $q->end_action_tr();

    # Input desired primaries
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Desired Primaries:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME      => "desired_primaries",
		      DEFAULT	    => "1",
		      SIZE      => 2,
		      MAXLENGTH => 2
		      });
    $q->end_action_td();
    $q->end_action_tr();

    # Input maximum primaries
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Maximum Primaries:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME      => "maximum_primaries",
		      DEFAULT	    => "1",
		      SIZE      => 2,
		      MAXLENGTH => 2
		      });
    $q->end_action_td();
    $q->end_action_tr();

    # Input automatic failback
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Automatic Failback:"));
    $q->end_action_td();
    $q->start_action_td();
    my (%truefalse) = (false => gettext("False"), true => gettext("True"));
    print $q->popup_menu({ NAME	=> "failback",
		      VALUES	=> ["true", "false"],
		      LABELS	=> \%truefalse,
		      DEFAULT => "false",
		      });
    $q->end_action_td();
    $q->end_action_tr();

    # Input pingpong interval
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Pingpong Interval:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME      => "pingpong",
		      VALUE	    => "3600",
		      SIZE      => 10,
		      });
    print gettext("Seconds");
    $q->end_action_td();
    $q->end_action_tr();

    # Dependencies
    $q->start_action_tr();
    $q->start_action_td({COlSPAN => 2});
    print $q->CGI::div({CLASS => "action-window-label-text"}, 
	gettext("Dependencies:"));
    $q->end_action_td();
    $q->end_action_tr();

    # Input rg dependencies
    $q->start_action_tr();
    $q->start_action_td({ VALIGN => "top" });
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Resource Groups:"));
    $q->end_action_td();
    $q->start_action_td();

    print $q->checkbox_group({ NAME      => "rg_dependencies",
			       VALUES    => \@rglist,
			       LINEBREAK => "true"
			       });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Global Resources Used:"));
    $q->end_action_td();
    $q->start_action_td();
    my (%allnone) = (all => gettext("All"), none => gettext("None"));
    print $q->popup_menu({ NAME => "globalresources",
		      VALUES	=> ["all", "none"],
		      LABELS	=> \%allnone,
		      DEFAULT=> "all",
		      });
    $q->end_action_td();
    $q->end_action_tr();

    # Input network dependencies
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Implicit Network Dependencies:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->popup_menu({ NAME	=> "netdependencies",
		      VALUES	=> ["true", "false"],
		      LABELS	=> \%truefalse,
		      DEFAULT	=> "true",
		      });
    $q->end_action_td();
    $q->end_action_tr();

    # End the table
    $q->end_action_table();

    # Start the action button table
    $q->start_action_button_table();
    $q->start_action_tr({ HEIGHT  => 15 });
    $q->start_action_td({ COLSPAN => 2 });
    print $q->img({ SRC    => "/images/dot.gif",
		    ALT    => "",
		    HEIGHT => 1,
		    WIDTH  => 1
		    });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr({ HEIGHT  => 1 });
    $q->start_action_td({ COLSPAN => 2,
			  CLASS => "action-line-row"
			  });
    print $q->img({ SRC    => "/images/dot.gif",
		    ALT    => "",
		    HEIGHT => 1
		    });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr({ HEIGHT  => 6 });
    $q->start_action_td({ COLSPAN => 2,
			  HEIGHT  => 5
			  });
    print $q->img({ SRC    => "/images/dot.gif",
		    ALT    => "",
		    HEIGHT => 6
		    });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr();
    $q->start_action_td({ ALIGN => "RIGHT",
			  WIDTH => "80%" 
			  });
    print $q->input({ TYPE => "SUBMIT",
		      NAME => "doit",
                      onclick => "bundleListAsInput(this.form.selnodes,this.form.primaries_list)",
		      VALUE => gettext(" Create Resource Group ")
		      });
    $q->end_action_td();
    $q->start_action_td({ ALIGN => "RIGHT",
			  WIDTH => "20%" 
			  });
    print $q->input({ TYPE => "BUTTON",
		      VALUE => gettext(" Cancel "),
		      ONCLICK => "history.back()"
		      });
    $q->end_action_tr();
    $q->end_action_td();

    # End the table
    $q->end_action_table();

    # End the form
    print $q->end_form();
} else {
    # Start the form
    print $q->start_form({ ACTION => "/cgi-bin/rg/rg_add.pl",
			   METHOD => "POST"
			   });

    # Start the action table
    $q->start_action_table();

    # Input rg name
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-label-text"}, 
	gettext("Resource Group Name:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME => "rg",
		      SIZE => 10
		      });
    $q->end_action_td();
    $q->end_action_tr();

    # Input primaries
    $q->start_action_tr();
    $q->start_action_td({ VALIGN => "top" });
    print $q->CGI::div({CLASS => "action-window-label-text"}, 
	gettext("Primaries List:"));
    $q->end_action_td();

    $q->start_action_td();

	# print "<br> -- WIDGET START SIMPLE 1 --<br>";

    print $q->start_table({ BORDER => 0 });
    print $q->start_Tr();
    print $q->start_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Available Nodes"));
    print $q->end_td();

    print $q->start_td();
    print $q->end_td();

    print $q->start_td();
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	gettext("Selected Nodes"));
    print $q->end_td();
    print $q->end_Tr();

    print $q->start_Tr();
    print $q->start_td();
    # The following hidden input carries data from the selected nodes list
    print $q->input ({ type => "hidden",
		       name => "primaries_list" });

    print $q->scrolling_list({ single => "true",
			       size => "9",
			       name => "availnodes",
			       values => \@availnodeslist,
			       labels => \%nodelistlabels
			     });
    print $q->end_td();
    print $q->start_td();

    print $q->input({ type => "button",
		      value => gettext("   Add >>  "),
		      onclick => "horizontal_shift(this.form.availnodes, this.form.selnodes);
				  bundleListAsInput(this.form.selnodes,this.form.primaries_list)" });
		      print $q->br();
    print $q->input({ type => "button",
		      value => gettext("<< Remove"),
		      onclick => "horizontal_shift(this.form.selnodes, this.form.availnodes);
				  bundleListAsInput(this.form.selnodes,this.form.primaries_list)" });
    print $q->end_td();
    print $q->start_td();
    print $q->scrolling_list({ single => "true",
			       size => "9",
			       name => "selnodes",
			       values => \@selectednodeslist,
			       labels => \%nodelistlabels
			     });
    print $q->end_td();

    print $q->start_td();

    print $q->input({ type => "button",
		      value => gettext("  Up  "),
		      onclick => "vertical_shift(this.form.selnodes,0);
				  bundleListAsInput(this.form.selnodes,this.form.primaries_list)" });
		      print $q->br();
    print $q->input({ type => "button",
		      value => gettext("Down"),
		      onclick => "vertical_shift(this.form.selnodes,1);
				  bundleListAsInput(this.form.selnodes,this.form.primaries_list)" });
    print $q->end_td();

    print $q->end_Tr();
    print $q->end_table();

	# print "<br> -- WIDGET SIMPLE END --<br>";

    $q->end_action_td();
    $q->end_action_tr();


    # Input desired primaries
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-label-text"}, 
	gettext("Desired Primaries:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME      => "desired_primaries",
		      VALUE	    => "1",
		      SIZE      => 2,
		      MAXLENGTH => 2
		      });
    $q->end_action_td();
    $q->end_action_tr();

    # Input maximum primaries
    $q->start_action_tr();
    $q->start_action_td();
    print $q->CGI::div({CLASS => "action-window-label-text"}, 
	gettext("Maximum Primaries:"));
    $q->end_action_td();
    $q->start_action_td();
    print $q->textfield({ NAME      => "maximum_primaries",
		      VALUE	    => "1",
		      SIZE      => 2,
		      MAXLENGTH => 2
		      });
    $q->end_action_td();
    $q->end_action_tr();

    # End the table
    $q->end_action_table();

    # Start the action button table
    $q->start_action_button_table();
    $q->start_action_tr({ HEIGHT  => 15 });
    $q->start_action_td({ COLSPAN => 2 });
    print $q->img({ SRC    => "/images/dot.gif",
		    ALT    => "",
		    HEIGHT => 1,
		    WIDTH  => 1
		    });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr({ HEIGHT  => 1 });
    $q->start_action_td({ COLSPAN => 2,
			  CLASS => "action-line-row"
			  });
    print $q->img({ SRC    => "/images/dot.gif",
		    ALT    => "",
		    HEIGHT => 1
		    });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr({ HEIGHT  => 6 });
    $q->start_action_td({ COLSPAN => 2,
			  HEIGHT  => 5
			  });
    print $q->img({ SRC    => "/images/dot.gif",
		    ALT    => "",
		    HEIGHT => 6
		    });
    $q->end_action_td();
    $q->end_action_tr();

    $q->start_action_tr();
    $q->start_action_td({ ALIGN => "RIGHT",
			  WIDTH => "80%" 
			  });
    print $q->input({ TYPE => "SUBMIT",
		      NAME => "full",
                      onclick => "bundleListAsInput(this.form.selnodes,this.form.primaries_list)",
		      VALUE => gettext(" Configure Additional Properties ")
		      });
    print "&nbsp;";
    print $q->input({ TYPE => "SUBMIT",
		      NAME => "doit",
                      onclick => "bundleListAsInput(this.form.selnodes,this.form.primaries_list)",
		      VALUE => gettext(" Create Resource Group ")
		      });
    $q->end_action_td();
    $q->start_action_td({ ALIGN => "RIGHT",
			  WIDTH => "20%" 
			  });
    print $q->input({ TYPE => "BUTTON",
		      VALUE => gettext(" Cancel "),
		      ONCLICK => "history.back()"
		      });
    $q->end_action_tr();
    $q->end_action_td();

    # End the table
    $q->end_action_table();

    # End the form
    print $q->input({ TYPE  => "HIDDEN",
		      NAME  => "func",
		      VALUE => "create"
		      });
    print $q->end_form();
}

# End the html document
$rg_obj->footer($q);
